"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StructureApi = exports.supportsStructureV2 = exports.makeStructure = exports.convertToStructureV2 = exports.getStructureFileVersion = exports.getStoryboard = exports.getStartSlide = exports.getSlideSubslides = exports.getFlatChapterSlides = exports.getChapterSlides = exports.getChapter = exports.getVisibleChapters = exports.getHiddenChapters = exports.getChapters = exports.getFlatSlide = exports.getSlide = exports.getArchivedSlides = exports.getFlatHiddenSlides = exports.getHiddenSlides = exports.getFlatVisibleSlides = exports.getVisibleSlides = exports.getFlatSlides = exports.getSlidesAsync = exports.getSlides = void 0;
const path_1 = require("path");
const interfaces_1 = require("./types/interfaces");
Object.defineProperty(exports, "StructureApi", { enumerable: true, get: function () { return interfaces_1.StructureApi; } });
const helpers_1 = require("./utils/helpers");
const utils_1 = require("./utils");
const semver_1 = require("semver");
function getAllSlidesAsNested(structureV2Json, slideTypes, systemSettings) {
    const allChaptersId = Object.keys(structureV2Json.chapters);
    const slideIds = (0, helpers_1.getOrderedSlidesId)(allChaptersId, structureV2Json);
    Object.keys(structureV2Json.slides).forEach((slideId) => {
        if (!slideIds.includes(slideId)) {
            slideIds.push(slideId);
        }
    });
    const filtersList = (0, utils_1.getFiltersList)(slideTypes);
    const slides = slideIds.reduce((acc, slideId) => {
        const slideType = (0, helpers_1.getSlideType)(structureV2Json, slideId);
        if (filtersList?.includes(slideType) || !filtersList?.length) {
            const slide = getSlide(structureV2Json, slideId, systemSettings);
            if (slide) {
                acc.push(slide);
            }
        }
        return acc;
    }, []);
    return slides;
}
const getSlides = (structureJson, slideTypes, systemSettings) => {
    const structureV2Json = convertToStructureV2(structureJson);
    const slides = getAllSlidesAsNested(structureV2Json, slideTypes, systemSettings);
    return slides.filter(slide => !(0, utils_1.isSubslide)(slides, slide.id));
};
exports.getSlides = getSlides;
const getSlidesAsync = async (structureJson, slideTypes, systemSettings, callback) => {
    const slides = getSlides(structureJson, slideTypes, systemSettings);
    const modifiedSlides = [];
    if (callback) {
        for await (const slideData of slides) {
            const modifiedSubslides = [];
            if (slideData.subslides) {
                for await (const subslideData of slideData.subslides) {
                    const subslideCopy = Object.assign({}, subslideData);
                    const modifiedSubslide = await callback(subslideCopy);
                    modifiedSubslides.push(modifiedSubslide);
                }
                slideData.subslides = modifiedSubslides;
            }
            const slideCopy = Object.assign({}, slideData);
            modifiedSlides.push(await callback(slideCopy));
        }
        return modifiedSlides;
    }
    return slides;
};
exports.getSlidesAsync = getSlidesAsync;
const getFlatSlides = (structureJson, slideTypes, systemSettings) => {
    const structureV2Json = convertToStructureV2(structureJson);
    const nestedSlides = getAllSlidesAsNested(structureV2Json, slideTypes, systemSettings);
    return nestedSlides.map(nestedSlide => {
        const { id, name, chapter, type, thumbnail, template } = nestedSlide;
        const parentSlide = (0, helpers_1.getParentSlide)(structureV2Json, nestedSlide.id);
        return {
            id,
            name,
            chapter,
            type,
            thumbnail,
            template,
            isSubslide: !!parentSlide,
            parentSlide,
            custom: (0, utils_1.getSlideCustomFields)(structureV2Json, id)
        };
    });
};
exports.getFlatSlides = getFlatSlides;
const getVisibleSlides = (structureJson, systemSettings) => {
    const structureV2Json = convertToStructureV2(structureJson);
    return getSlides(structureV2Json, { visible: true, archived: false, hidden: false }, systemSettings);
};
exports.getVisibleSlides = getVisibleSlides;
const getFlatVisibleSlides = (structureJson, systemSettings) => {
    const structureV2Json = convertToStructureV2(structureJson);
    return getFlatSlides(structureV2Json, { visible: true, archived: false, hidden: false }, systemSettings);
};
exports.getFlatVisibleSlides = getFlatVisibleSlides;
const getHiddenSlides = (structureJson, systemSettings) => {
    const structureV2Json = convertToStructureV2(structureJson);
    const allHiddenSlides = getSlides(structureV2Json, { visible: false, archived: false, hidden: true }, systemSettings);
    return allHiddenSlides;
};
exports.getHiddenSlides = getHiddenSlides;
const getFlatHiddenSlides = (structureJson, systemSettings) => {
    const structureV2Json = convertToStructureV2(structureJson);
    return getFlatSlides(structureV2Json, { visible: false, archived: false, hidden: true }, systemSettings);
};
exports.getFlatHiddenSlides = getFlatHiddenSlides;
const getArchivedSlides = (structureJson, systemSettings) => {
    const structureV2Json = convertToStructureV2(structureJson);
    return getSlides(structureV2Json, { visible: false, archived: true, hidden: false }, systemSettings);
};
exports.getArchivedSlides = getArchivedSlides;
const getSlide = (structureJson, slideId, systemSettings) => {
    const structureV2Json = convertToStructureV2(structureJson);
    const slideChapter = (0, helpers_1.getSlideChapter)(structureV2Json, slideId);
    return {
        id: slideId,
        name: structureV2Json.slides[slideId].name,
        chapter: {
            id: slideChapter,
            name: slideChapter ? structureV2Json.chapters[slideChapter].name : undefined,
            type: slideChapter ? (0, helpers_1.getChapterType)(structureV2Json, slideChapter) : undefined,
            storyboardIndex: slideChapter ? structureV2Json.storyboard.findIndex(chapter => (0, utils_1.normalizeChapter)(chapter).chapter === slideChapter) : undefined,
            custom: slideChapter ? (0, utils_1.getChapterCustomFields)(structureJson, slideChapter) : {}
        },
        type: (0, helpers_1.getSlideType)(structureV2Json, slideId),
        thumbnail: systemSettings ? (0, path_1.join)(systemSettings.path.slides, slideId, systemSettings.path.slide.thumbnail) : undefined,
        template: structureV2Json.slides[slideId].template,
        subslides: (0, helpers_1.getSubslidesNestedData)(structureV2Json, slideId),
        custom: (0, utils_1.getSlideCustomFields)(structureV2Json, slideId)
    };
};
exports.getSlide = getSlide;
const getFlatSlide = (structureJson, slideId, systemSettings) => {
    const structureV2Json = convertToStructureV2(structureJson);
    const parentSlide = (0, helpers_1.getParentSlide)(structureV2Json, slideId);
    const NestedSlide = getSlide(structureV2Json, slideId, systemSettings);
    if (NestedSlide) {
        const { id, name, chapter, type, thumbnail, template } = NestedSlide;
        return {
            id,
            name,
            chapter,
            type,
            thumbnail,
            template,
            isSubslide: !!parentSlide,
            parentSlide: parentSlide,
            custom: (0, utils_1.getSlideCustomFields)(structureV2Json, id)
        };
    }
    return;
};
exports.getFlatSlide = getFlatSlide;
const getChapters = (structureJson, chapterTypes) => {
    const structureV2Json = convertToStructureV2(structureJson);
    const chapterIdList = Object.keys(structureV2Json.chapters);
    const result = chapterIdList.map((chapterId) => getChapter(structureV2Json, chapterId));
    if (chapterTypes) {
        const filterOptions = (0, utils_1.getFiltersList)(chapterTypes);
        return result.filter(slide => slide.type && filterOptions?.includes(slide.type));
    }
    return result;
};
exports.getChapters = getChapters;
const getHiddenChapters = (structureJson) => {
    const structureV2Json = convertToStructureV2(structureJson);
    return getChapters(structureV2Json, { visible: false, hidden: true });
};
exports.getHiddenChapters = getHiddenChapters;
const getVisibleChapters = (structureJson) => {
    const structureV2Json = convertToStructureV2(structureJson);
    return getChapters(structureV2Json, { visible: true, hidden: false });
};
exports.getVisibleChapters = getVisibleChapters;
const getChapter = (structureJson, chapterId) => {
    const structureV2Json = convertToStructureV2(structureJson);
    return {
        id: chapterId,
        name: structureV2Json.chapters[chapterId].name,
        type: (0, helpers_1.getChapterType)(structureV2Json, chapterId),
        custom: (0, utils_1.getChapterCustomFields)(structureV2Json, chapterId)
    };
};
exports.getChapter = getChapter;
const getChapterSlides = (structureJson, chapterId, slideTypes, systemSettings) => {
    const structureV2Json = convertToStructureV2(structureJson);
    const filters = { visible: slideTypes?.visible, hidden: slideTypes?.hidden, archived: false };
    const allSlides = getSlides(structureV2Json, filters, systemSettings);
    return allSlides
        .filter(slide => slide.chapter?.id === chapterId)
        .flatMap(({ id, name, chapter, type, thumbnail, template, subslides }) => {
        return {
            id,
            name,
            chapter,
            type,
            thumbnail,
            template,
            subslides,
            custom: (0, utils_1.getSlideCustomFields)(structureV2Json, id)
        };
    });
};
exports.getChapterSlides = getChapterSlides;
const getFlatChapterSlides = (structureJson, chapterId, slideTypes, systemSettings) => {
    const structureV2Json = convertToStructureV2(structureJson);
    const ChapterNestedSlides = getChapterSlides(structureJson, chapterId, slideTypes, systemSettings);
    return ChapterNestedSlides.reduce((acc, nestedSlide) => {
        const parentSlide = (0, helpers_1.getParentSlide)(structureV2Json, nestedSlide.id);
        const { id, name, chapter, template, thumbnail, type } = nestedSlide;
        acc.push({ id, name, chapter, template, thumbnail, type, isSubslide: !!parentSlide, parentSlide: parentSlide, custom: (0, utils_1.getSlideCustomFields)(structureV2Json, nestedSlide.id) });
        if (nestedSlide.subslides && nestedSlide.subslides.length) {
            const flatSubslides = nestedSlide.subslides
                .map(subslide => getFlatSlide(structureJson, subslide.id, systemSettings))
                .filter(subslide => subslide);
            acc.push(...flatSubslides);
        }
        return acc;
    }, []);
};
exports.getFlatChapterSlides = getFlatChapterSlides;
const getSlideSubslides = (structureJson, slideId, slideTypes, systemSettings) => {
    const structureV2Json = convertToStructureV2(structureJson);
    const subslides = (0, helpers_1.getSubslidesNestedData)(structureV2Json, slideId, systemSettings);
    const filters = (0, utils_1.getFiltersList)(slideTypes);
    if (filters && Array.isArray(filters)) {
        return subslides.filter(slide => filters.includes(slide.type));
    }
    return subslides;
};
exports.getSlideSubslides = getSlideSubslides;
const getStartSlide = (structureJson, systemSettings) => {
    const structureV2Json = convertToStructureV2(structureJson);
    if (structureV2Json.start?.slide || structureV2Json.start?.chapter) {
        if (structureV2Json.start.slide && !structureV2Json.start.chapter) {
            return {
                slide: structureV2Json.start.slide,
                chapter: (0, helpers_1.getSlideChapter)(structureV2Json, structureV2Json.start.slide),
            };
        }
        if (structureV2Json.start.chapter && !structureV2Json.start.slide) {
            return {
                slide: getChapterSlides(structureJson, structureV2Json.start.chapter, { visible: true }, systemSettings)[0]?.id,
                chapter: structureV2Json.start.chapter,
            };
        }
        return structureV2Json.start;
    }
    const firstVisibleSlide = getVisibleSlides(structureV2Json, systemSettings)[0];
    return { slide: firstVisibleSlide.id, chapter: firstVisibleSlide.chapter?.id };
};
exports.getStartSlide = getStartSlide;
const getStoryboard = (structureJson, chapterTypes, systemSettings) => {
    const structureV2Json = convertToStructureV2(structureJson);
    const storyboardChapters = structureV2Json.storyboard.map(chapterEntity => (0, utils_1.normalizeChapter)(chapterEntity).chapter);
    const filters = (0, utils_1.getFiltersList)(chapterTypes);
    const storyboardChaptersData = storyboardChapters.map(chapterId => getChapter(structureV2Json, chapterId));
    const filteredChapters = filters ? storyboardChaptersData.filter(chapter => chapter.type && filters.includes(chapter.type)) : storyboardChaptersData;
    return filteredChapters.map((chapterData) => {
        const slides = getChapterSlides(structureV2Json, chapterData.id, { visible: true, archived: true, hidden: true }, systemSettings).map(slide => {
            return { ...slide, subslides: getSlideSubslides(structureV2Json, slide.id, undefined, systemSettings) };
        });
        const nestedSlideData = {
            ...chapterData,
            slides
        };
        return nestedSlideData;
    });
};
exports.getStoryboard = getStoryboard;
const getStructureFileVersion = (structureJson) => {
    const json = structureJson;
    return json.structureFileVersion ?? 1;
};
exports.getStructureFileVersion = getStructureFileVersion;
const convertToStructureV2 = (structureJson) => {
    if (getStructureFileVersion(structureJson) === 2) {
        return structureJson;
    }
    const initialJson = JSON.parse(JSON.stringify(structureJson));
    const resultingJson = {
        structureFileVersion: 2,
        slides: {},
        chapters: {},
        storyboard: [],
        archivedSlides: [],
        start: initialJson.start,
    };
    const normalizedSlides = (0, utils_1.normalizeStructureV1Subslides)(initialJson.slides);
    const normalizedSlidesWithSubslides = Object
        .entries(normalizedSlides)
        .filter(([slideName, slideData]) => !!slideData.subslides);
    const normalizedChapters = (0, utils_1.normalizeStructureV1Chapters)(initialJson);
    const convertedStoryboard = (0, helpers_1.convertStoryboardStructureV1ToV2)(initialJson.storyboard);
    const convertedChapters = (0, helpers_1.migrateSubslidesToChapters)(normalizedSlidesWithSubslides, normalizedChapters, initialJson);
    const convertedSlides = (0, helpers_1.removeSubslidesFromNormalizedSlides)(normalizedSlides);
    resultingJson.storyboard = convertedStoryboard;
    resultingJson.chapters = convertedChapters || normalizedChapters;
    resultingJson.slides = convertedSlides;
    const archivedSlides = getSlides(resultingJson, { archived: true });
    resultingJson.archivedSlides = archivedSlides.map(slideData => ({ slide: slideData.id, chapter: slideData.chapter?.id }));
    return resultingJson;
};
exports.convertToStructureV2 = convertToStructureV2;
const makeStructure = (nestedSlidesData, targetStructureVersion) => {
    const structureScheme = {
        slides: {},
        chapters: {},
        storyboard: [],
    };
    (0, helpers_1.restoreSlidesStructure)(nestedSlidesData, structureScheme);
    const nestedChapters = (0, helpers_1.restoreChaptersStructure)(nestedSlidesData, structureScheme);
    (0, helpers_1.restoreStoryboard)(nestedChapters, structureScheme);
    return targetStructureVersion === 1 ? structureScheme : convertToStructureV2(structureScheme);
};
exports.makeStructure = makeStructure;
const supportsStructureV2 = (ewizardjsVersion) => {
    const versionWithSupportedHiddenSlides = '5.22.0';
    return (0, semver_1.lte)(versionWithSupportedHiddenSlides, ewizardjsVersion);
};
exports.supportsStructureV2 = supportsStructureV2;
//# sourceMappingURL=index.js.map